%
% Filename:    virtex4_ml403_postgeneration.m
%
% Description: Defines board-specific parameters (e.g. ucf file, 
%              non-memory mapped ports) before invoking the 
%              generic JTAG post-generation callback function.

function st = virtex4_ml403_postgeneration(params)

  % Device position in the boundary scan chain (beginning at 1).
  params.('boundary_scan_position') = '2';

  % Instruction register length of every scan chain device. 
  params.('instruction_register_lengths') = '[8, 10, 8]';

  % Constraints file to use for this compilation target.
  params.('ucf_template') = 'virtex4_ml403.ucf'; 

  % Define non-memory mapped ports.
  non_mm_ports.('LEDs') = {'out', 4};
  non_mm_ports.('audio_sdata_in') = {'in', 1};
  non_mm_ports.('audio_bit_clk') = {'in', 1};
  non_mm_ports.('lcd_e') = {'out', 1};
  non_mm_ports.('lcd_rs') = {'out', 1};
  non_mm_ports.('lcd_rw') = {'out', 1};
  non_mm_ports.('lcd_db') = {'out', 4};
  non_mm_ports.('vga_b') = {'out', 5};
  non_mm_ports.('vga_psave_n') = {'out', 1};
  non_mm_ports.('vga_blank_n') = {'out', 1};
  non_mm_ports.('vga_sync_n') = {'out', 1};
  non_mm_ports.('vga_r') = {'out', 5};
  non_mm_ports.('vga_g') = {'out', 5};
  non_mm_ports.('vga_hsync') = {'out', 1};
  non_mm_ports.('vga_vsync') = {'out', 1};
  non_mm_ports.('vga_clk') = {'out', 1};
  non_mm_ports.('audio_sdata_out') = {'out', 1};
  non_mm_ports.('audio_reset_n') = {'in', 1};
  params.('non_memory_mapped_ports') = non_mm_ports;

  % You may use your own top-level netlist file by uncommenting the 
  % following line and setting the 'vendor_toplevel' field accordingly.
  % params.('vendor_toplevel') = 'virtex4_ml403_toplevel';

  % If you use your own top-level, you must tell SysGen what netlist 
  % files are required.  Set the 'vendor_netlists' field to a cell 
  % array listing the required file names. 
  % params.('vendor_netlists') = {'virtex4_ml403_toplevel.ngc','virtex4_ml403.edf'};

  % Invoke the JTAG post-generation callback function to run
  % Xilinx tools and create a run-time co-simulation token.
  try
    st = xlJTAGPostGeneration(params);
  catch
    errordlg(['-- An unknown error was encountered while running ' ...
             'the JTAG hardware co-simulation flow for the ' ...
             'Virtex4 ML403']);
    st = 1;
  end
